#!inc/bin/testml-cpan

# XXX Skipping this test for now. Might need a %Skip in TestML
'1' == '1'
# *grammar.compile.optimize.yaml.clean == *yaml


=== Question Mark Expansion
--- SKIP
--- grammar
a: /(:foo)/
--- yaml
a:
  .rgx: /(?:foo)/
